﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Algorithms.DataAnalysis.Distances;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("EuclidianNorm", "A norm function that uses Euclidean distance")]
  public class EuclidianDistance : DistanceBase<IEnumerable<double>> {

    #region HLConstructors & Boilerplate
    [StorableConstructor]
    protected EuclidianDistance(bool deserializing) : base(deserializing) { }
    protected EuclidianDistance(EuclidianDistance original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) { return new EuclidianDistance(this, cloner); }
    public EuclidianDistance() { }
    #endregion

    #region statics
    public static double GetDistance(double[] point1, double[] point2) {
      if (point1.Length != point2.Length) throw new ArgumentException("Euclidean distance not defined on vectors of different length");
      return Math.Sqrt(point1.Zip(point2, (a1, b1) => (a1 - b1) * (a1 - b1)).Sum());
    }
    public static double GetDistance(IEnumerable<double> a, IEnumerable<double> b, double threshold) {
      double sum = 0;
      var it1 = a.GetEnumerator();
      var it2 = b.GetEnumerator();
      while (sum > threshold * threshold && it1.MoveNext() && it2.MoveNext()) {
        var d = it1.Current - it2.Current;
        sum += d * d;
      }
      it1.Dispose();
      it2.Dispose();
      return sum;
    }
    #endregion

    public override double Get(IEnumerable<double> a, IEnumerable<double> b) {
      return GetDistance(a.ToArray(), b.ToArray());
    }
    public override double Get(IEnumerable<double> a, IEnumerable<double> b, double threshold) {
      return GetDistance(a, b, threshold);
    }
  }
}
